
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Handler = function(overlay, doc)
{
	this.overlay = overlay ? overlay : null;
	this.element = null;
	this.timeout = null;
	this.outline = null;
	this.doc = doc ? doc : null;
	this.pages = null;
	this.address = null;
	this.status = null;
	this.handlers = [];

	this.start = function(doc, pages, mode)
	{
		if (doc) this.doc = doc;

		this.pages = pages;
		this.address = yarip.getAddress(this.doc, mode);
		this.status = this.doc.defaultView.status;
		var active = false;

		if (this.doc && this.doc.nodeName == "#document" && this.pages && this.address)
		{
			this.doc.addEventListener("mouseover", this, false);
			this.doc.addEventListener("mouseout", this, false);
			this.doc.addEventListener("mousedown", this, false);
			this.doc.addEventListener("keypress", this, false);
			active = true;
		}

		if (this.handlers.length > 0) for (var i = 0; i < this.handlers.length; i++) if (this.handlers[i].start(null, this.pages, mode)) active = true;

		if (this.doc.defaultView) this.doc.defaultView.focus();

		return active;
	}

	this.stop = function()
	{
		if (!this.doc || !this.pages) return;

		this.doc.removeEventListener("mouseover", this, false);
		this.doc.removeEventListener("mouseout", this, false);
		this.doc.removeEventListener("mousedown", this, false);
		this.doc.removeEventListener("keypress", this, false);

		if (this.handlers.length > 0) for (var i = 0; i < this.handlers.length; i++) this.handlers[i].stop();

		this.reset();
	}

	this.addHandler = function(handler)
	{
		this.handlers[this.handlers.length] = handler;
	}

	this.handleEvent = function(event)
	{
		if (event.target.localName.match(/^(frameset|frame|iframe)$/i)) return;

		switch (event.type)
		{
			case "mouseover":
				if (this.element) this.reset();
				this.element = event.target;
				if (!this.element.href) this.switchWithAnchorAncestor();
				this.timeout = setTimeout(this.select, 100, this);
				event.stopPropagation();
				break;

			case "mouseout":
				if (!this.element) break;
				this.reset();
				event.stopPropagation();
				break;

			case "mousedown":
				switch (event.button)
				{
					case 0: // left mouse button (remove and remember)
						this.remove();
						event.stopPropagation();
						break;

					case 1: // middle mouse button (remove temporary)
						this.remove(true);
						event.stopPropagation();
						break;

					default: // escape
						this.overlay.stop();
				}
				break;

			case "keypress":
				switch (event.keyCode)
				{
					case 27: // escape
						this.overlay.stop();
						event.stopPropagation();
						break;

					case 0: // character
						switch (event.charCode)
						{
							case 114: // character `r' (remove and remember)
								this.remove();
								event.stopPropagation();
								break;

							case 116: // character `t' (remove temporary)
								this.remove(true);
								event.stopPropagation();
						}
				}
		}
	}

	this.switchWithAnchorAncestor = function()
	{
		if (!this.element.localName.match(/^(abbr|acronym|b|basefont|bdo|big|br|button|cite|code|del|dfn|em|font|i|img|ins|kbd|label|map|q|samp|select|small|span|strong|sub|sup|textarea|tt|u|var)$/i)) return;

		if (!this.element.xValue) this.element.xValue = yarip.calculateXPath(this.element);

		var xValue = this.element.xValue;
		if (xValue.match(/^[/]{2}/)) xValue = yarip.calculateXPath(this.element, true);
		var expr = xValue.match(/^.*?\/a\b[^/]*/i);
		if (!expr) return;

		var elements = yarip.getElementsByXPath(this.doc, expr[0]);
		if (!elements) return;

		this.element = elements.snapshotItem(0);
		var shortPath = yarip.calculateXPath(this.element, false);
		this.element.xValue = shortPath ? shortPath : expr;
	}

	this.select = function(handler)
	{
		if (!handler) handler = this;

		if (!handler.element) return;

		handler.outline = handler.element.style.outline;
		if (handler.element.getAttribute("status") == "whitelisted") handler.element.style.outline = "3px solid #990000";
		else handler.element.style.outline = "3px solid #000099";

		if (!handler.element.xValue) handler.element.xValue = yarip.calculateXPath(handler.element);
		if (handler.doc && handler.doc.defaultView) handler.doc.defaultView.status = handler.element.xValue;
	}

	this.remove = function(removeTemporary)
	{
		if (!this.element) return;

		if (!this.element.xValue) this.element.xValue = yarip.calculateXPath(this.element);

		var removed = false;
		if (removeTemporary) removed = yarip.removeByXPath(this.doc, this.pages, this.address, new yarip.XPath(this.element.xValue, this.overlay.deep), null, true);
		else
		{
			removed = yarip.removeByXPath(this.doc, this.pages, this.address, new yarip.XPath(this.element.xValue, this.overlay.deep), true);
			this.overlay.changesMade = this.overlay.changesMade || removed;
		}

		this.reset(removed);
	}

	this.reset = function(removed)
	{
		if (this.timeout) clearTimeout(this.timeout);
		else this.timeout = null;

		if (!this.element) return;

		if (!removed) this.element.style.outline = this.outline;

		this.outline = null;
		this.element = null;

		content.document.defaultView.status = "";
	}
}
