
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Loader = function(overlay)
{
	this.overlay = overlay;
	this.found = false;

	this.load = function(doc, pages)
	{
		if (!doc || doc.nodeName != "#document" || !pages) return false;

		var address = yarip.getAddress(doc, yarip.MODES.page);
		if (!address) return false;

		var aAddress = yarip.getAddressArray(doc, pages);
		if (this.doWhitelisting(doc, address, aAddress, pages)) this.doBlacklisting(doc, address, aAddress, pages);
		var found = this.found;
		this.found = false;
		return found;
	}

	this.doWhitelisting = function(doc, pageAddress, aAddress, pages)
	{
		var hasNoBlacklist = true;

		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			for (var xValue in pages[aAddress[i]].whitelist.xpaths)
			{
				if (!pages[aAddress[i]].whitelist.xpaths[xValue].deep && pageAddress != aAddress[i]) continue;

				var elements = yarip.getElementsByXPath(doc, xValue);
				if (elements) pages[aAddress[i]].whitelist.xpaths[xValue].found++; // increment `found'
				else
				{
					pages[aAddress[i]].whitelist.xpaths[xValue].notFound++; // increment `not found'
					continue;
				}

				this.found = true;
				hasNoBlacklist = hasNoBlacklist && pages[aAddress[i]].blacklist.length == 0;

				for (var j = 0; j < elements.snapshotLength; j++)
				{
					var element = elements.snapshotItem(j);
					while (element && element.nodeType == 1 && element.getAttribute("status") != "whitelisted")
					{
						element.setAttribute("status", "whitelisted");
						element = element.parentNode;
					}
				}
			}
		}

		if (this.found && hasNoBlacklist)
		{
			yarip.removeByXPath(doc, pages, pageAddress, new yarip.XPath("//*[not(@status='whitelisted')]")); // remove all except whitelisted
			var elements = yarip.getElementsByXPath(doc, "//*[@status='whitelisted']");
			if (!elements) return false;

			doc.body.removeAttribute("background");
			doc.body.style.margin = "0";

			for (var i = 0; i < elements.snapshotLength; i++)
			{
				var element = elements.snapshotItem(i);
				element.style.background = null;
				if (!element.attributes) continue;

				var tag = element.localName.toLowerCase();
				if (!tag.match(/div|table|td|th/)) continue;

				for (var j in element.attributes)
				{
					if (!element.attributes[j]) continue;

					var n = element.attributes[j].name;
					switch (n)
					{
						case "align":
							element.setAttribute(n, "center");
							break;

						case "border":
						case "cellpadding":
						case "cellspacing":
							element.setAttribute(n, "0");
							break;

						case "background":
						case "bgcolor":
						case "colspan":
						case "height":
						case "rowspan":
						case "style":
							element.removeAttribute(n);
							break;

						case "valign":
							element.setAttribute(n, "top");
					}
				}
			}

			return false;
		}

		return true;
	}

	this.doBlacklisting = function(doc, pageAddress, aAddress, pages)
	{
		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			for (var xValue in pages[aAddress[i]].blacklist.xpaths)
			{
				var xpath = pages[aAddress[i]].blacklist.xpaths[xValue];

				if (!xpath.deep && this.checkDeep(pageAddress, aAddress[i])) continue;

				if (yarip.removeByXPath(doc, pages, aAddress[i], xpath))
				{
					this.found = this.found || true;
					pages[aAddress[i]].blacklist.xpaths[xValue].found++; // increment `found'
				}
				else pages[aAddress[i]].blacklist.xpaths[xValue].notFound++; // increment `not found'
			}
		}
	}

	this.checkDeep = function(pageAddress, address)
	{
		if (address.match(yarip.domainRegExp))
		{
			var domain = pageAddress.match(yarip.domainRegExp);
			if (domain) return domain[0] != address;
		}
		return pageAddress != address;
	}
}
