
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Overlay = function()
{
	this.enabled = false;
	this.active = false;
	this.noFlicker = false;
	this.changesMade = false;

	this.enabledObserver = null;
	this.modeObserver = null;
	this.reloadObserver = null;

	this.appcontent = null;
	this.contextMenu = null;
	this.panelContainer = null;

	this.doc = null;
	this.control = null;
	this.loader = null;
	this.handler = null;
	this.highlighter = null;
	this.pages = null;
	this.mode = null;
	this.deep = null;

	this.start = function(event)
	{
		if (event.button != 0) return;

		var doc = content.document;

		if (this.active)
		{
			this.stop(doc, false);
			return;
		}

		if (!this.enabled)
		{
			this.toggleEnabled(true);
			if (doc)
			{
				if (this.loader.load(doc, this.pages)) this.setStatus(doc, "found");
				if (doc.body) doc.body.setAttribute("status", "show");
			}
			return;
		}

		if (!doc) return;

		this.handler = null;
		if (!doc.defaultView.handler) return;
		this.handler = doc.defaultView.handler;

		if (!this.handler.start(doc, this.pages, this.mode))
		{
			this.stop(doc, false);
			return;
		}

		this.active = true;
		this.doc = doc;
		this.setStatus(doc, "started");
	}

	this.stop = function(doc, noStateChange)
	{
		if (!this.active || !this.handler) return;

		this.handler.stop();
		this.active = false;

		doc = doc ? doc : this.doc;
		if (!doc) return;

		if (!noStateChange) this.setStatus(doc, "stopped");

		this.doc = null;
	}

	this.save = function(force)
	{
		this.stop();
		if (!force && !this.changesMade) return;

		this.control.savePages(this.pages);

		this.changesMade = false;
		this.reloadObserver.notify();
	}

	this.reset = function(mode, noStateChange)
	{
		this.stop();
		this.toggleEnabled(true);

		var node = document.popupNode;
		if (!node) return;
		var doc = node.ownerDocument;
		if (!doc) return;
		if (typeof mode != "number")
		{
			var aAddress = yarip.getAddressArray(doc, this.pages);
			for (var i = aAddress.length - 1; i >= 0; i--) this.pages[aAddress[i]] = null;
			if (noStateChange) return;
			return;
		}

		var address = yarip.getAddress(doc, mode);
		if (!address || !this.pages || !this.pages[address]) return;
		this.pages[address] = null;
	}

	this.reload = function()
	{
		this.stop();
		this.toggleEnabled(true);

		this.pages = this.control.loadPages();

		this.changesMade = false;
		this.reloadObserver.notify();
	}

	this.whitelist = function(doc, node)
	{
		this.stop();
		this.toggleEnabled(true);

		if (!node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!doc || !address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for whitelisting", xValue);
		if (!xValue) return;

		var elements = yarip.getElementsByXPath(doc, xValue);
		if (elements)
		{
			if (!this.pages[address]) this.pages[address] = new yarip.Page(address);
			this.pages[address].whitelist.addXPath(new yarip.XPath(xValue, this.deep));
			this.changesMade = true;
		}
		// ERROR MESSAGE
	}

	this.blacklist = function(doc, node)
	{
		this.stop();
		this.toggleEnabled(true);

		if (!node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for blacklisting", xValue);
		if (!xValue) return;

		var found = yarip.removeByXPath(doc, this.pages, address, new yarip.XPath(xValue, this.deep), true);
		if (found) this.changesMade = true;
		// ERROR MESSAGE
	}

	this.startHighlighting = function(node)
	{
		if (!node) return;

		this.highlighter.element = node;
		this.highlighter.select();
	}

	this.stopHighlighting = function()
	{
		this.highlighter.reset();
	}

	this.setMode = function(mode, clear)
	{
		if (typeof mode != "number") return;

		this.stop();
		this.toggleEnabled(true);

		if (clear)
		{
			yarip.$("yarip-page-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-fqdn-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-sld-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-user-menuitem").setAttribute("checked", "false");
		}

		switch (mode)
		{
			case yarip.MODES.page:
				yarip.$("yarip-page-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.fqdn:
				yarip.$("yarip-fqdn-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.sld:
				yarip.$("yarip-sld-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.user:
				var doc = content.document;
				var address = yarip.getAddress(doc, this.mode);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				address = prompt("Edit address", address);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				yarip.address = address;
				yarip.$("yarip-user-menuitem").setAttribute("checked", "true");
		}

		this.mode = mode;

		this.modeObserver.notify(mode);
	}

	this.updateMode = function()
	{
		var mode = this.control.getValue(yarip.MODE, yarip.MODES.fqdn, yarip.INTEGER);
		if (mode != yarip.MODES.user) this.setMode(mode, true);
	}

	this.toggleEnabled = function(enabled)
	{
		var oldEnabled = this.control.getValue(yarip.ENABLED, true, yarip.BOOLEAN);
		if (enabled != true && enabled != false) enabled = !oldEnabled;
		this.enabled = enabled;

		if (!enabled)
		{
			this.stop(null, true);
			this.toggleNoFlicker(false);
		}

		if (enabled != oldEnabled) this.enabledObserver.notify(enabled);

		yarip.$("yarip-enabled-menuitem").setAttribute("checked", "" + enabled);
		this.setStatus(content.document, enabled ? "enabled" : "disabled");
	}

	this.updateEnabled = function()
	{
		var enabled = this.control.getValue(yarip.ENABLED, true, yarip.BOOLEAN);
		this.toggleEnabled(enabled);
	}

	this.toggleNoFlicker = function(noFlicker)
	{
		if (noFlicker == this.noFlicker) return;

		if (noFlicker != true && noFlicker != false) noFlicker = !this.control.getValue(yarip.NO_FLICKER, true, yarip.BOOLEAN);
		this.noFlicker = noFlicker && this.enabled;
		this.control.setValue(yarip.NO_FLICKER, this.noFlicker, yarip.BOOLEAN);

		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("chrome://yarip/content/inject.css", yarip.CHARSET, null); // URI, CHARSET, BASEURI
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);

		if (this.noFlicker) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	}

	this.toggleDeep = function(deep)
	{
		if (deep == this.deep) return;

		if (deep != true && deep != false) deep =  !this.deep;
		this.deep = deep;

		yarip.$("yarip-deep-menuitem").setAttribute("checked", "" + this.deep);
	}

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.control = new yarip.Control(this);
				this.pages = this.control.loadPages();
				this.loader = new yarip.Loader(this);
				this.highlighter = new yarip.Handler(this);

				this.enabledObserver = new yarip.Observer(
					yarip.ENABLED_ROOT,
					yarip.VALUE,
					function() { yarip.overlay.updateEnabled(); },
					function(enabled) { yarip.overlay.control.setValue(yarip.ENABLED, enabled, yarip.BOOLEAN); }
				);
				this.modeObserver = new yarip.Observer(
					yarip.MODE_ROOT,
					yarip.VALUE,
					function() { yarip.overlay.updateMode(); },
					function(mode) { yarip.overlay.control.setValue(yarip.MODE, mode, yarip.INTEGER); }
				);
				this.reloadObserver = new yarip.Observer(
					yarip.RELOAD_ROOT,
					yarip.VALUE,
					function() { yarip.overlay.reload(); },
					function() { yarip.overlay.control.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); }
				);

				this.appcontent = yarip.$("appcontent");
				this.contextMenu = yarip.$("contentAreaContextMenu");
				if (gBrowser) this.panelContainer = gBrowser.mPanelContainer;

				this.toggleEnabled(this.control.getValue(yarip.ENABLED, true, yarip.BOOLEAN));
				this.setMode(yarip.MODES.fqdn, true);
				this.toggleNoFlicker(this.control.getValue(yarip.NO_FLICKER, false, yarip.BOOLEAN));
				this.toggleDeep(true);

				if (this.appcontent) this.appcontent.addEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.addEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.addEventListener("select", this, false);
				window.removeEventListener("load", this, false);

				break;

			case "unload":
				this.save();

				this.enabledObserver.unregister();
				this.modeObserver.unregister();
				this.reloadObserver.unregister();

				if (this.appcontent) this.appcontent.removeEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.removeEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.removeEventListener("select", this, false);
				window.removeEventListener("load", this, false);
				window.removeEventListener("unload", this, false);
				window.removeEventListener("DOMMenuItemActive", this, false);
				window.removeEventListener("keypress", this, false);

				break;

			case "DOMContentLoaded":
				this.stop();

				this.toggleDeep(true);

				var doc = event.originalTarget;
				if (!doc) return;

				var inFrame = doc.defaultView.frameElement != null;

				// HANDLER
				if (!inFrame)
				{
					doc.defaultView.handler = new yarip.Handler(this);
					if (this.enabled) this.setStatus(doc, "stopped");
				}
				else
				{
					var rootDoc = doc;
					while (rootDoc.defaultView.frameElement) rootDoc = rootDoc.defaultView.frameElement.ownerDocument; // get root doc
					if (!rootDoc.defaultView.handler) break;
					var handler = new yarip.Handler(this, doc);
					rootDoc.defaultView.handler.addHandler(handler);
				}

				// LOADER
				if (doc.nodeName == "#document" && this.enabled && this.pages)
				{
					if (this.loader.load(doc, this.pages))
					{
						this.toggleNoFlicker(true);
						this.setStatus(doc, "found");
					}
					else if (!inFrame) this.toggleNoFlicker(false);

					if (this.mode == yarip.MODES.user) this.setMode(yarip.MODES.fqdn, true);
				}
				else this.toggleNoFlicker(false);

				if (doc.body) doc.body.setAttribute("status", "show");

				break;

			case "DOMMenuItemActive":
				this.toggleNoFlicker(false);
				break;

			case "keypress":
				if (event.ctrlKey) this.toggleNoFlicker(false);
				break;

			case "popupshowing":
				var node = document.popupNode;
				var doc = node.ownerDocument;

				var hidden = yarip.getAddress(doc, yarip.overlay.mode) == null;
				if (!hidden) yarip.$("yarip-element-menu").setAttribute("label", node.localName.toUpperCase());
				yarip.$("yarip-menu").hidden = hidden;
				yarip.$("yarip-menu-sep").hidden = hidden;

				var yaripFrameMenu = yarip.$("yarip-frame-menu");
				var inFrame = gContextMenu.inFrame;
				var disabled = false;
				if (inFrame)
				{
					var frame = node.ownerDocument.defaultView.frameElement;
					yaripFrameMenu.setAttribute("label", frame.localName.toUpperCase());
					disabled = yarip.getAddress(frame.ownerDocument, yarip.overlay.mode) == null;
				}
				yaripFrameMenu.hidden = !inFrame;
				if (disabled) yaripFrameMenu.setAttribute("disabled", true);
				else yaripFrameMenu.removeAttribute("disabled");

				break;

			case "select":
				this.setStatus(gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument);
		}
	}

	this.setStatus = function(doc, status)
	{
		if (this.enabled)
		{
			if (!status)
			{
				status = doc.defaultView.myStatus;
				switch (status)
				{
					case "enabled":
					case "stopped":
					case "found":
						break;

					case "disabled":
					case "started": if (this.active) break;
					default:
						status = "enabled";
				}
			}
		}
		else status = "disabled";

		if (doc.defaultView) doc.defaultView.myStatus = status;
		if (doc == gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument) yarip.$("yarip-statusbarpanel").setAttribute("status", status);
	}
}
